/*
###############################################################################
Include Part
###############################################################################
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>			// to use printf
#include <string.h>
#include <util/delay.h>
#include "serial.h" 			// serial related functions
#include "socket.h" 			
#include "types.h"
#include "w5200.h"
#include "lcd.h"
#include "spi.h"
#include "dhcp.h"
//#include "delay.h"

/*
###############################################################################
Define Part
###############################################################################
*/
#define	MAX_BUF_SIZE	2048		// Maximum receive buffer size

//#define __DEF_TCPS_LOOPBACK__
//#define __DEF_TCPC_LOOPBACK__
#define __DEF_UDP_LOOPBACK__
//#define __DEF_MACRAW_LOOPBACK__
//#define __DEF_WOL__
//#define __DEF_DHCP__
//#define __DEF_PPPOE_TEST__
#define __DEF_WINDOW_FULL__

/*
###############################################################################
Local  Variable Declaration Part
###############################################################################
*/
uint8 * data_buf = (uint8*) TX_BUF;	// Position of receive buffer
static u_char evb_lcd_text[LCD_MAX_ROW][LCD_MAX_COL+1];
void evb_set_lcd_text(u_char row, u_char* lcd);
#ifdef __DEF_DHCP__
extern u_char GET_SN_MASK[4];		/**< Subnet mask received from the DHCP server */
extern u_char GET_GW_IP[4];		/**< Gateway IP address received from the DHCP server */
extern u_char GET_DNS_IP[4];		/**< DNS server IP address received from the DHCP server */
extern u_char GET_SIP[4];		/**< Leased source IP address received from the DHCP server */
#endif
#ifdef __DEF_WINDOW_FULL__
uint8 test_ch;
uint8 test_data[10] = { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99 };
#endif

/*
###############################################################################
Function Prototype Declaration Part
###############################################################################
*/
void InitMCU(void); 	
void mcu_init(void);	
void loopback_tcps(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode);
#ifdef __DEF_TCPC_LOOPBACK__
void loopback_tcpc(SOCKET s, uint8* addr, uint16 port, uint8 * data_buf, uint16 mode);
#endif
void loopback_udp(SOCKET s, uint16 port, uint8* data_buf, uint16 mode);
#ifdef __DEF_MACRAW_LOOPBACK__
void loopback_macraw(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode);
#endif
void evb_soft_reset(void);

/*
###############################################################################
Function Implementation Part
###############################################################################
*/

int main(void)
{
#ifdef __DEF_DHCP__
	uint8 i;
#endif
#ifdef __DEF_IINCHIP_DBG__  
	uint16 j;
#endif
	uint8 tmp[6] = {0,0,0,0,0,0};
	uint8 ip[4] = {192,168,1,86};                   // for setting SIP register
	uint8 gw[4] = {192,168,1,1};                     // for setting GAR register
	uint8 sn[4] = {255,255,255,0};                     // for setting SUBR register
	uint8 mac[6] = {0x00,0x08,0xDC,0x11,0x22,0x86};      // for setting SHAR register

#ifdef __DEF_TCPC_LOOPBACK__
	uint8 serverip[4] = {222,98,173,216};
#endif

  	uint8 txsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
  	uint8 rxsize[MAX_SOCK_NUM] = {2,2,2,2,2,2,2,2};
  
	InitMCU();		
	IINCHIP_SpiInit0();

	IINCHIP_WRITE((SHAR0 + 0),mac[0]);
	IINCHIP_WRITE((SHAR0 + 1),mac[1]);
	IINCHIP_WRITE((SHAR0 + 2),mac[2]);
	IINCHIP_WRITE((SHAR0 + 3),mac[3]);
	IINCHIP_WRITE((SHAR0 + 4),mac[4]);
	IINCHIP_WRITE((SHAR0 + 5),mac[5]);

#ifdef __DEF_DHCP__
	init_dhcp_client(0, evb_soft_reset, evb_soft_reset);
	if(!getIP_DHCPS())
	{
		printf("Fail to get a IP adress from DHCP server\r\n");
		printf("Apply the default network information!!!\r\n");
		close(0);
	}
	else
	{
		for(i=0; i<4; i++)
		{
			ip[i] = GET_SIP[i];
		}
		for(i=0; i<4; i++)
		{
			gw[i] = GET_GW_IP[i];
		}
		for(i=0; i<4; i++)
		{
			sn[i] = GET_SN_MASK[i];
		}
		printf("Get network information from DHCP Server...\r\n");	
		check_DHCP_state(0);
	}
	IINCHIP_WRITE((GAR0 + 0),gw[0]);
	IINCHIP_WRITE((GAR0 + 1),gw[1]);
	IINCHIP_WRITE((GAR0 + 2),gw[2]);
	IINCHIP_WRITE((GAR0 + 3),gw[3]);
	
	IINCHIP_WRITE((SUBR0 + 0),sn[0]);
	IINCHIP_WRITE((SUBR0 + 1),sn[1]);
	IINCHIP_WRITE((SUBR0 + 2),sn[2]);
	IINCHIP_WRITE((SUBR0 + 3),sn[3]);
	
	IINCHIP_WRITE((SIPR0 + 0),ip[0]);
	IINCHIP_WRITE((SIPR0 + 1),ip[1]);
	IINCHIP_WRITE((SIPR0 + 2),ip[2]);
	IINCHIP_WRITE((SIPR0 + 3),ip[3]);

#else
	IINCHIP_WRITE((GAR0 + 0),gw[0]);
	IINCHIP_WRITE((GAR0 + 1),gw[1]);
	IINCHIP_WRITE((GAR0 + 2),gw[2]);
	IINCHIP_WRITE((GAR0 + 3),gw[3]);
	
	IINCHIP_WRITE((SUBR0 + 0),sn[0]);
	IINCHIP_WRITE((SUBR0 + 1),sn[1]);
	IINCHIP_WRITE((SUBR0 + 2),sn[2]);
	IINCHIP_WRITE((SUBR0 + 3),sn[3]);
	
	IINCHIP_WRITE((SIPR0 + 0),ip[0]);
	IINCHIP_WRITE((SIPR0 + 1),ip[1]);
	IINCHIP_WRITE((SIPR0 + 2),ip[2]);
	IINCHIP_WRITE((SIPR0 + 3),ip[3]);
#endif
  	sysinit(txsize,rxsize);

#if (__DEF_IINCHIP_BUS__ == __DEF_IINCHIP_INDIRECT_MODE__)
	printf("=================================================\r\n");
	printf("   W5200 Indirect mode  Net Config Information\r\n");
	printf("=================================================\r\n");
#else 
	printf("============================================\r\n");
	printf("   W5200 SPI mode  Net Config Information\r\n");
	printf("============================================\r\n");
#endif	

	printf("MAC ADDRESS      : \r\n");

	tmp[0] = IINCHIP_READ(SHAR0+0);
	tmp[1] = IINCHIP_READ(SHAR0+1);
	tmp[2] = IINCHIP_READ(SHAR0+2);
	tmp[3] = IINCHIP_READ(SHAR0+3);
	tmp[4] = IINCHIP_READ(SHAR0+4);
	tmp[5] = IINCHIP_READ(SHAR0+5);

	printf("%02x.%02x.%02x.%02x.%02x.%02x\r\n",tmp[0],tmp[1],tmp[2],tmp[3],tmp[4],tmp[5]);	

	printf("SUBNET MASK      : \r\n"); 

	tmp[0] = IINCHIP_READ(SUBR0 + 0);
	tmp[1] = IINCHIP_READ(SUBR0 + 1);
	tmp[2] = IINCHIP_READ(SUBR0 + 2);
	tmp[3] = IINCHIP_READ(SUBR0 + 3);
	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);

	printf("G/W IP ADDRESS   : \r\n");
	
	tmp[0] = IINCHIP_READ(GAR0 + 0);
	tmp[1] = IINCHIP_READ(GAR0 + 1);
	tmp[2] = IINCHIP_READ(GAR0 + 2);
	tmp[3] = IINCHIP_READ(GAR0 + 3);
	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);

	printf("LOCAL IP ADDRESS : \r\n"); 
	
	tmp[0] = IINCHIP_READ(SIPR0 + 0);
	tmp[1] = IINCHIP_READ(SIPR0 + 1);
	tmp[2] = IINCHIP_READ(SIPR0 + 2);
	tmp[3] = IINCHIP_READ(SIPR0 + 3);
	
	printf("%d.%d.%d.%d\r\n",tmp[0],tmp[1],tmp[2],tmp[3]);
	printf("====================================\r\n");

#ifdef __DEF_IINCHIP_INT__	
	IINCHIP_WRITE(IMR, 0xFF);
	IINCHIP_WRITE(IMR2, 0xF0);
#endif
#ifdef __DEF_WOL__
	IINCHIP_WRITE(MR, MR_WOL);
	printf("MR = %02x\r\n",IINCHIP_READ(MR));
#endif
#ifdef __DEF_PPPOE_TEST__
	IINCHIP_WRITE(MR, MR_PPPOE);
	
	IINCHIP_WRITE(Sn_DHAR0(0),0x00);
	IINCHIP_WRITE(Sn_DHAR0(0)+1,0x08);
	IINCHIP_WRITE(Sn_DHAR0(0)+2,0xDC);
	IINCHIP_WRITE(Sn_DHAR0(0)+3,0x11);
	IINCHIP_WRITE(Sn_DHAR0(0)+4,0x22);
	IINCHIP_WRITE(Sn_DHAR0(0)+5,0xD8);
	
	IINCHIP_WRITE(Sn_DIPR0(0),0xDE);
	IINCHIP_WRITE(Sn_DIPR0(0)+1,0x62);
	IINCHIP_WRITE(Sn_DIPR0(0)+2,0xAD);
	IINCHIP_WRITE(Sn_DIPR0(0)+3,0xD8);

	IINCHIP_WRITE(Sn_DPORT0(0),0x0B);
	IINCHIP_WRITE(Sn_DPORT0(0)+1,0xB8);

	IINCHIP_WRITE(Sn_MSSR0(0),0x05);
	IINCHIP_WRITE(Sn_MSSR0(0)+1,0x78);
	
	socket(0,Sn_MR_PPPOE,3000,0);
	close(0);

	while(1)
	{	
		loopback_tcpc(1,serverip,5000,data_buf,0);
	}
#else
	while(1)
	{
#ifdef __DEF_MACRAW_LOOPBACK__
		loopback_macraw(0,5000,data_buf,0);
#else
		loopback_tcps(0,5000,data_buf,0);
#endif
      loopback_tcps(1,5001,data_buf,0);
      loopback_tcps(2,5002,data_buf,0);
      loopback_tcps(3,5003,data_buf,0);
      loopback_tcps(4,5004,data_buf,0);
//      loopback_tcps(5,5005,data_buf,0);
//      loopback_tcps(6,5006,data_buf,0);
//      loopback_tcps(7,5007,data_buf,0);
//      loopback_tcpc(7,serverip,5000,data_buf,0);                        

      loopback_udp(5,3000,data_buf,0);
      loopback_udp(6,3001,data_buf,0);
      loopback_udp(7,3002,data_buf,0);
   	}
#endif   	
}

/*
********************************************************************************
*              AVR128 Initialization Function
*
* Description: 
* Arguments  : None.
* Returns    : None.
* Note       : 
********************************************************************************
*/
void evb_set_lcd_text(u_char row, u_char* lcd)
{
	if(row < LCD_MAX_ROW && strlen((char*)lcd) <= LCD_MAX_COL)
	{
		memset(evb_lcd_text[row],' ',LCD_MAX_COL);
		evb_lcd_text[row][LCD_MAX_COL] = 0;
		memcpy(evb_lcd_text[row],lcd,strlen((char*)lcd));
		lcd_gotoxy(0,row);
		lcd_puts((char*)evb_lcd_text[row]);
	}
}

void InitMCU(void)
{ 
	mcu_init();
	lcd_init();
	evb_set_lcd_text(0,(u_char*)"W5200 Test");
	uart_init(0,7);
}

void mcu_init(void) 
{
	cli();
		
#ifdef __DEF_IINCHIP_INT__	
	EICRA=0x00;
	EICRB=0x00;
	EIMSK=0x00;
	EIFR=0x00;
#else
	EICRA = 0x00;			// External Interrupt Control Register A clear
	EICRB = 0x02;			// External Interrupt Control Register B clear // edge 
	EIMSK = 0x20;		// External Interrupt Mask Register : 0x10
	EIFR = 0xFF;			// External Interrupt Flag Register all clear
	DDRE &= ~0x20;		// Set PE Direction 
	PORTE |= 0x20;		// Set PE Default value
#endif


#if (ATMEGA128_NUM_WAIT == ATMEGA128_0WAIT)
	MCUCR = 0x80;		
	XMCRA=0x40;
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_1WAIT)
	MCUCR = 0xc0;		// MCU control regiseter : enable external ram
	XMCRA=0x40;		// External Memory Control Register A : 
						// Low sector   : 0x1100 ~ 0x7FFF
						// Upper sector : 0x8000 ~ 0xFFFF
#elif (ATMEGA128_NUM_WAIT == ATMEGA128_2WAIT )
	MCUCR = 0x80;
	XMCRA=0x42;
#elif ((ATMEGA128_NUM_WAIT == ATMEGA128_3WAIT)
	MCUCR = 0xc0;
	XMCRA=0x42;
#else
#error "unknown atmega128 number wait type"
#endif	

	sei();				// enable interrupts
}


#ifdef __DEF_TCPS_LOOPBACK__
void loopback_tcps(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode)
{
	uint16 len,idx = 0;
	static uint8 lastdata=0x00; 
	switch(getSn_SR(s))
	{
		case SOCK_ESTABLISHED:						// If client is connected 
			if (getSn_IR(s) & Sn_IR_CON) 			// Confirm if received the data
			{
          	printf("Connect OK : %d\r\n",s);
          	IINCHIP_WRITE(Sn_IR(s),Sn_IR_CON);	// clear Sn_IR_CON
        	}
#if 1
			if((len=getSn_RX_RSR(s)) > 0)				// Handle as much as buffer size first when receiving data more than system buffer
			{
#ifdef __DEF_IINCHIP_DBG__				
				printf("rsr_len = %d\r\n",len);
#endif					

#if 1						
				if (len > TX_RX_MAX_BUF_SIZE) len = TX_RX_MAX_BUF_SIZE;
#ifdef __DEF_IINCHIP_DBG__
				printf("rsr_len = %d\r\n",len);
#endif
				len = recv(s, data_buf, len);     // recv
				if(len !=send(s, data_buf, len))  // send
     			{
					printf("Send Fail\r\n");
				}
#if 0
				if(lastdata != data_buf[0])
				{
					printf("first data = %02x, last data = %02x\r\n",data_buf[0],lastdata);
					printf("BLOCK LOSS\r\n");
					while(1);
				}
				else if((uint8)(data_buf[0]+len) != (uint8)(data_buf[len-1]+1))
				{
					printf("expected = %02x, real = %02x\r\n",(uint8)(data_buf[0]+len),(uint8)(data_buf[len-1]+1));
					printf("DATA INVALID\r\n");
					while(1);
				}
				lastdata = (uint8)(data_buf[len-1]+1);
				send(s, data_buf, len);
#endif
#endif
			}
#endif
			break;
		case SOCK_CLOSE_WAIT:                           		// If the client request to close connection and wait for closing successfully
			printf("CLOSE_WAIT : %d\r\n",s);					
			disconnect(s);
			break;
		case SOCK_CLOSED:                                               // If the socket connected to client already has been closed
			printf("CLOSED. CH : %d\r\n",s);	
			close(s);                        // close the SOCKET
      	socket(s,Sn_MR_TCP,port,mode);   // open the SOCKET
			break;
		case SOCK_INIT:                     // The SOCKET opened with TCP mode
      	listen(s);                       // listen to any connection request from "TCP CLIENT"
      	printf("LOOPBACK_TCPSStarted. CH : %d\r\n",s);
      	break;
   	default:
      	break;
	}
}
#endif

#ifdef __DEF_WINDOW_FULL__
void loopback_tcps(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode)
{
	uint16 len,idx = 0;
	static uint8 lastdata=0x00; 
	switch(getSn_SR(s))
	{
		case SOCK_ESTABLISHED:						// If client is connected 
			if (getSn_IR(s) & Sn_IR_CON) 			// Confirm if received the data
			{
          	printf("Connect OK : %d\r\n",s);
          	IINCHIP_WRITE(Sn_IR(s),Sn_IR_CON);	// clear Sn_IR_CON
        	}
//        	test_ch = getch();
  			_delay_ms(10);
			while(1)
  			{
  				send(s, test_data, 10);
  				_delay_us(100);
			}
			break;
		case SOCK_CLOSE_WAIT:                           		// If the client request to close connection and wait for closing successfully
			printf("CLOSE_WAIT : %d\r\n",s);					
			disconnect(s);
			break;
		case SOCK_CLOSED:                                               // If the socket connected to client already has been closed
			printf("CLOSED. CH : %d\r\n",s);	
			close(s);                        // close the SOCKET
      	socket(s,Sn_MR_TCP,port,mode);   // open the SOCKET
			break;
		case SOCK_INIT:                     // The SOCKET opened with TCP mode
      	listen(s);                       // listen to any connection request from "TCP CLIENT"
      	printf("LOOPBACK_TCPSStarted. CH : %d\r\n",s);
      	break;
   	default:
      	break;
	}
}
#endif

#ifdef __DEF_TCPC_LOOPBACK__
void loopback_tcpc(SOCKET s, uint8* addr, uint16 port, uint8 * data_buf, uint16 mode)
{
	uint16 len;
	static uint16 any_port = 1000;
	switch(getSn_SR(s))
	{
	 	case SOCK_ESTABLISHED:              // ESTABLISHED?
         		if(getSn_IR(s) & Sn_IR_CON)      // check Sn_IR_CON bit
         		{
            			printf("%d : Connect OK\r\n",s);
            			IINCHIP_WRITE(Sn_IR(s),Sn_IR_CON);	// clear Sn_IR_CON
         		}
         		if((len=getSn_RX_RSR(s)) > 0)    // check the size of received data
         		{
            			len = recv(s,data_buf,len);        // recv
            			if(len !=send(s,data_buf,len))     // send
            			{
               				printf("%d : Send Fail.len = %d\r\n",s,len);
            			}
         		}
        			break;
                                          // ---------------
   		case SOCK_CLOSE_WAIT:                  // PASSIVE CLOSED
         		disconnect(s);                   // disconnect 
         		break;
                                          // --------------
   		case SOCK_CLOSED:                      // CLOSED
      			close(s);                           // close the SOCKET
      			socket(s,Sn_MR_TCP,any_port++,mode);   // open the SOCKET with TCP mode and any source port number
					printf("socket init OK!\r\n");
      			break;
                		                       // ------------------------------
   		case SOCK_INIT:                        // The SOCKET opened with TCP mode
      			connect(s, addr, port);             // Try to connect to "TCP SERVER"
      			printf("%d : LOOPBACK_TCPC Started %02x.%02x.%02x.%02x:%02x\r\n:",s,addr[0],addr[1],addr[2],addr[3],port);
      			break;
   		default:
      			break;
	}
}
#endif

#ifdef __DEF_UDP_LOOPBACK__
void loopback_udp(SOCKET s, uint16 port, uint8* data_buf, uint16 mode)
{
	uint16 len;
	uint8 destip[4];
	uint16 destport;	
  
 	switch(getSn_SR(s))
  	{
  		case SOCK_UDP:             
			if((len=getSn_RX_RSR(s)) > 0)                   // check the size of received data
         {
            len = recvfrom(s,data_buf,len,destip,&destport);  // receive data from a destination 
            if(len != sendto(s,data_buf,len,destip,destport))  // send the data to the destination
            {
              	printf("Sendto Fail\r\n");
            }
         }
			break;                                                   
  		case SOCK_CLOSED:                                  // CLOSED
         close(s);                                       // close the SOCKET
         socket(s,Sn_MR_UDP,port,mode);                  // open the SOCKET with UDP mode
         break;
  		default:
         break;
  }
}
#endif

#ifdef __DEF_MACRAW_LOOPBACK__
void loopback_macraw(SOCKET s, uint16 port, uint8 * data_buf, uint16 mode)
{
	uint16 len, send_cnt = 0;
	uint8 serverip[4] = {192,168,1,85};
	
 	switch(getSn_SR(s))
  	{
  		case SOCK_MACRAW:	
			if((len=getSn_RX_RSR(s)) > 0)                   // check the size of received data
         {
            len = recvfrom(s,data_buf,len,0,0);  // receive data from a destination 
            if(((data_buf[0] == 0x00) && (data_buf[1] == 0x08) && (data_buf[2] == 0xDC) &&
            	(data_buf[3] == 0x11) && (data_buf[4] == 0x22) && (data_buf[5] == 0x86)) ||
            	((data_buf[0] == 0xFF) && (data_buf[1] == 0xFF) && (data_buf[2] == 0xFF) &&
            	(data_buf[3] == 0xFF) && (data_buf[4] == 0xFF) && (data_buf[5] == 0xFF)))
            {
					printf("MAC RAW msg arrived\r\n");
					printf("dest mac=%.2X.%.2X.%.2X.%.2X.%.2X.%.2X\r\n",data_buf[0],data_buf[1],data_buf[2],data_buf[3],data_buf[4],data_buf[5]);
					printf("src  mac=%.2X.%.2X.%.2X.%.2X.%.2X.%.2X\r\n",data_buf[6],data_buf[7],data_buf[8],data_buf[9],data_buf[10],data_buf[11]);
					printf("type    =%.2X%.2X\r\n",data_buf[12],data_buf[13]); 
					send_cnt = send_cnt + 1;
				}
				if(send_cnt == 10000)
				{
            	if(len != sendto(s,data_buf,len,serverip,3000))  // send the data to the destination
            	{
            	  	printf("Sendto Fail\r\n");
            	}
            	send_cnt = 0;
            }
         }
			break;                                                   
  		case SOCK_CLOSED:                                  // CLOSED
         close(s);                                       // close the SOCKET
         socket(s,Sn_MR_MACRAW,port,mode);                  // open the SOCKET with UDP mode
         break;
  		default:
         break;
  }            
}
#endif

#ifdef __DEF_DHCP__
void evb_soft_reset(void)
{
	printf("System Reset.\r\n");
	asm volatile("jmp 0x0000");	
}
#endif



